@@ -9,10 +9,57 @@
     - Secure Messaging Integrity (MAC) data format and padding
     - Secure Messaging Confidentiality encryption method and padding
 
+CVN module combines application cryptogram generation, key derivation and
+secure messaging in one class per CVN.
+
+    >>> import pyemv
+    >>> cvn18 = pyemv.cvn.VisaCVN18(
+    ...     iss_mk_ac=bytes.fromhex('AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA'),
+    ...     iss_mk_smi=bytes.fromhex('BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB'),
+    ...     iss_mk_smc=bytes.fromhex('CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC'),
+    ...     pan='1234567890123456',
+    ...     psn='00')
+
+    >>> atc = bytes.fromhex('0FFF')
+    >>> arqc = cvn18.generate_ac(
+    ...     tag_9f02=bytes.fromhex('000000009999'),
+    ...     tag_9f03=bytes.fromhex('000000000000'),
+    ...     tag_9f1a=bytes.fromhex('0840'),
+    ...     tag_95=bytes.fromhex('8000048000'),
+    ...     tag_5f2a=bytes.fromhex('0840'),
+    ...     tag_9a=bytes.fromhex('991231'),
+    ...     tag_9c=bytes.fromhex('01'),
+    ...     tag_9f37=bytes.fromhex('52BF4585'),
+    ...     tag_82=bytes.fromhex('1800'),
+    ...     tag_9f36=atc,
+    ...     tag_9f10=bytes.fromhex('06011203A0B800'))
+    >>> arqc.hex().upper()
+    '769577B5ABE9FE62'
+
+    >>> arpc = cvn18.generate_arpc(
+    ...     tag_9f26=arqc,
+    ...     tag_9f36=atc,
+    ...     csu=bytes.fromhex('00000000'))
+    >>> arpc.hex().upper()
+    '76503F48'
+
+    >>> command_mac = cvn18.generate_command_mac(
+    ...         command_header=bytes.fromhex('8418000008'),
+    ...         tag_9f26=arqc,
+    ...         tag_9f36=atc)
+    >>> command_mac.hex().upper()
+    'B5CB29759F9C3919'
+
+    >>> pin_command = cvn18.generate_pin_change_command(
+    ...         pin='9999',
+    ...         tag_9f26=arqc,
+    ...         tag_9f36=atc)
+    >>> pin_command.hex().upper()
+    '84240002182DC7A061323BA62472BC5308BD291B5F665B3A927E60661E'
 """
 
 
-from typing import Optional, Union
+import typing as _typing
 
 from pyemv import ac as _ac
 from pyemv import kd as _kd
@@ -74,8 +121,8 @@ class VisaCVN10(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: Union[bytes, str],
-        psn: Optional[Union[bytes, str]] = None,
+        pan: _typing.Union[bytes, str],
+        psn: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
@@ -189,7 +236,7 @@ class VisaCVN10(object):
         return _ac.generate_arpc_1(self._derive_sk_ac_none(), tag_9f26, arpc_rc)
 
     def _derive_sk_sm_visa(self, icc_mk_sm: bytes, tag_9f36: bytes) -> bytes:
-        """Derive Secure Messaging Session Key using Visa method
+        """Derive Secure Messaging Session Key using Visa method.
 
         Parameters
         ----------
@@ -214,7 +261,7 @@ class VisaCVN10(object):
         tag_9f36: bytes,
         command_data: bytes = b"",
     ) -> bytes:
-        r"""Message Authentication Code (MAC) for Secure Messaging Integrity
+        r"""Message Authentication Code (MAC) for Secure Messaging Integrity.
 
         Parameters
         ----------
@@ -239,7 +286,7 @@ class VisaCVN10(object):
         )
 
     def encrypt_command_data(self, command_data: bytes, tag_9f36: bytes) -> bytes:
-        """Command Data Encryption for Secure Messaging Confidentiality
+        """Command Data Encryption for Secure Messaging Confidentiality.
 
         Parameters
         ----------
@@ -263,10 +310,10 @@ class VisaCVN10(object):
 
     def generate_pin_change_command(
         self,
-        pin: Union[bytes, str],
+        pin: _typing.Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
-        current_pin: Optional[Union[bytes, str]] = None,
+        current_pin: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> bytes:
         """Generate a PIN change command with encrypted PIN block and MAC.
 
@@ -350,8 +397,8 @@ class VisaCVN18(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: Union[bytes, str],
-        psn: Optional[Union[bytes, str]] = None,
+        pan: _typing.Union[bytes, str],
+        psn: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option B.
@@ -447,7 +494,7 @@ class VisaCVN18(object):
         tag_9f26: bytes,
         tag_9f36: bytes,
         csu: bytes,
-        proprietary_auth_data: Optional[bytes] = None,
+        proprietary_auth_data: _typing.Optional[bytes] = None,
     ) -> bytes:
         """Generate Authorisation Response Cryptogram (ARPC) using method 2.
         Method for the generation of a 4-byte ARPC consists of applying
@@ -484,7 +531,7 @@ class VisaCVN18(object):
         )
 
     def _derive_sk_sm_visa(self, icc_mk_sm: bytes, tag_9f36: bytes) -> bytes:
-        """Derive Secure Messaging Session Key using Visa method
+        """Derive Secure Messaging Session Key using Visa method.
 
         Parameters
         ----------
@@ -509,7 +556,7 @@ class VisaCVN18(object):
         tag_9f36: bytes,
         command_data: bytes = b"",
     ) -> bytes:
-        r"""Message Authentication Code (MAC) for Secure Messaging Integrity
+        r"""Message Authentication Code (MAC) for Secure Messaging Integrity.
 
         Parameters
         ----------
@@ -535,7 +582,7 @@ class VisaCVN18(object):
         )
 
     def encrypt_command_data(self, command_data: bytes, tag_9f36: bytes) -> bytes:
-        """Command Data Encryption for Secure Messaging Confidentiality
+        """Command Data Encryption for Secure Messaging Confidentiality.
 
         Parameters
         ----------
@@ -559,10 +606,10 @@ class VisaCVN18(object):
 
     def generate_pin_change_command(
         self,
-        pin: Union[bytes, str],
+        pin: _typing.Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
-        current_pin: Optional[Union[bytes, str]] = None,
+        current_pin: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> bytes:
         """Generate a PIN change command with encrypted PIN block and MAC.
 
@@ -647,8 +694,8 @@ class InteracCVN133(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: Union[bytes, str],
-        psn: Optional[Union[bytes, str]] = None,
+        pan: _typing.Union[bytes, str],
+        psn: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
@@ -781,7 +828,7 @@ class InteracCVN133(object):
         )
 
     def _derive_sk_sm_common(self, icc_mk_sm: bytes, tag_9f26: bytes) -> bytes:
-        """Derive Secure Messaging Session Key using Common method
+        """Derive Secure Messaging Session Key using Common method.
 
         Parameters
         ----------
@@ -805,7 +852,7 @@ class InteracCVN133(object):
         tag_9f26: bytes,
         command_data: bytes = b"",
     ) -> bytes:
-        r"""Message Authentication Code (MAC) for Secure Messaging Integrity
+        r"""Message Authentication Code (MAC) for Secure Messaging Integrity.
 
         Parameters
         ----------
@@ -829,7 +876,7 @@ class InteracCVN133(object):
         )
 
     def encrypt_command_data(self, command_data: bytes, tag_9f26: bytes) -> bytes:
-        """Command Data Encryption for Secure Messaging Confidentiality
+        """Command Data Encryption for Secure Messaging Confidentiality.
 
         Parameters
         ----------
@@ -852,7 +899,7 @@ class InteracCVN133(object):
         )
 
     def generate_pin_change_command(
-        self, pin: Union[bytes, str], tag_9f26: bytes
+        self, pin: _typing.Union[bytes, str], tag_9f26: bytes
     ) -> bytes:
         """Generate a PIN change command with encrypted PIN block and MAC.
 
@@ -932,8 +979,8 @@ class MasterCardCVN16(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: Union[bytes, str],
-        psn: Optional[Union[bytes, str]] = None,
+        pan: _typing.Union[bytes, str],
+        psn: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
@@ -1065,7 +1112,7 @@ class MasterCardCVN16(object):
         return _ac.generate_arpc_1(self._derive_sk_arpc_none(), tag_9f26, arpc_rc)
 
     def _derive_sk_sm_common(self, icc_mk_sm: bytes, tag_9f26: bytes) -> bytes:
-        """Derive Secure Messaging Session Key using Common method
+        """Derive Secure Messaging Session Key using Common method.
 
         Parameters
         ----------
@@ -1090,7 +1137,7 @@ class MasterCardCVN16(object):
         tag_9f36: bytes,
         command_data: bytes = b"",
     ) -> bytes:
-        r"""Message Authentication Code (MAC) for Secure Messaging Integrity
+        r"""Message Authentication Code (MAC) for Secure Messaging Integrity.
 
         Parameters
         ----------
@@ -1121,7 +1168,7 @@ class MasterCardCVN16(object):
         )
 
     def encrypt_command_data(self, command_data: bytes, tag_9f26: bytes) -> bytes:
-        r"""Command Data Encryption for Secure Messaging Confidentiality
+        r"""Command Data Encryption for Secure Messaging Confidentiality.
 
         Parameters
         ----------
@@ -1150,7 +1197,7 @@ class MasterCardCVN16(object):
 
     def generate_pin_change_command(
         self,
-        pin: Union[bytes, str],
+        pin: _typing.Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
@@ -1241,8 +1288,8 @@ class MasterCardCVN17(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: Union[bytes, str],
-        psn: Optional[Union[bytes, str]] = None,
+        pan: _typing.Union[bytes, str],
+        psn: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
@@ -1385,7 +1432,7 @@ class MasterCardCVN17(object):
         return _ac.generate_arpc_1(self._derive_sk_arpc_none(), tag_9f26, arpc_rc)
 
     def _derive_sk_sm_common(self, icc_mk_sm: bytes, tag_9f26: bytes) -> bytes:
-        """Derive Secure Messaging Session Key using Common method
+        """Derive Secure Messaging Session Key using Common method.
 
         Parameters
         ----------
@@ -1410,7 +1457,7 @@ class MasterCardCVN17(object):
         tag_9f36: bytes,
         command_data: bytes = b"",
     ) -> bytes:
-        r"""Message Authentication Code (MAC) for Secure Messaging Integrity
+        r"""Message Authentication Code (MAC) for Secure Messaging Integrity.
 
         Parameters
         ----------
@@ -1441,7 +1488,7 @@ class MasterCardCVN17(object):
         )
 
     def encrypt_command_data(self, command_data: bytes, tag_9f26: bytes) -> bytes:
-        r"""Command Data Encryption for Secure Messaging Confidentiality
+        r"""Command Data Encryption for Secure Messaging Confidentiality.
 
         Parameters
         ----------
@@ -1470,7 +1517,7 @@ class MasterCardCVN17(object):
 
     def generate_pin_change_command(
         self,
-        pin: Union[bytes, str],
+        pin: _typing.Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
@@ -1561,8 +1608,8 @@ class MasterCardCVN20(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: Union[bytes, str],
-        psn: Optional[Union[bytes, str]] = None,
+        pan: _typing.Union[bytes, str],
+        psn: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
@@ -1684,7 +1731,7 @@ class MasterCardCVN20(object):
         )
 
     def _derive_sk_sm_common(self, icc_mk_sm: bytes, tag_9f26: bytes) -> bytes:
-        """Derive Secure Messaging Session Key using Common method
+        """Derive Secure Messaging Session Key using Common method.
 
         Parameters
         ----------
@@ -1709,7 +1756,7 @@ class MasterCardCVN20(object):
         tag_9f36: bytes,
         command_data: bytes = b"",
     ) -> bytes:
-        r"""Message Authentication Code (MAC) for Secure Messaging Integrity
+        r"""Message Authentication Code (MAC) for Secure Messaging Integrity.
 
         Parameters
         ----------
@@ -1740,7 +1787,7 @@ class MasterCardCVN20(object):
         )
 
     def encrypt_command_data(self, command_data: bytes, tag_9f26: bytes) -> bytes:
-        r"""Command Data Encryption for Secure Messaging Confidentiality
+        r"""Command Data Encryption for Secure Messaging Confidentiality.
 
         Parameters
         ----------
@@ -1769,7 +1816,7 @@ class MasterCardCVN20(object):
 
     def generate_pin_change_command(
         self,
-        pin: Union[bytes, str],
+        pin: _typing.Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
@@ -1860,8 +1907,8 @@ class MasterCardCVN21(object):
         iss_mk_ac: bytes,
         iss_mk_smi: bytes,
         iss_mk_smc: bytes,
-        pan: Union[bytes, str],
-        psn: Optional[Union[bytes, str]] = None,
+        pan: _typing.Union[bytes, str],
+        psn: _typing.Optional[_typing.Union[bytes, str]] = None,
     ) -> None:
         # Derive AC, SMI, and SMC ICC Master Keys for a new card
         # using option A.
@@ -1994,7 +2041,7 @@ class MasterCardCVN21(object):
         )
 
     def _derive_sk_sm_common(self, icc_mk_sm: bytes, tag_9f26: bytes) -> bytes:
-        """Derive Secure Messaging Session Key using Common method
+        """Derive Secure Messaging Session Key using Common method.
 
         Parameters
         ----------
@@ -2019,7 +2066,7 @@ class MasterCardCVN21(object):
         tag_9f36: bytes,
         command_data: bytes = b"",
     ) -> bytes:
-        r"""Message Authentication Code (MAC) for Secure Messaging Integrity
+        r"""Message Authentication Code (MAC) for Secure Messaging Integrity.
 
         Parameters
         ----------
@@ -2050,7 +2097,7 @@ class MasterCardCVN21(object):
         )
 
     def encrypt_command_data(self, command_data: bytes, tag_9f26: bytes) -> bytes:
-        r"""Command Data Encryption for Secure Messaging Confidentiality
+        r"""Command Data Encryption for Secure Messaging Confidentiality.
 
         Parameters
         ----------
@@ -2079,7 +2126,7 @@ class MasterCardCVN21(object):
 
     def generate_pin_change_command(
         self,
-        pin: Union[bytes, str],
+        pin: _typing.Union[bytes, str],
         tag_9f26: bytes,
         tag_9f36: bytes,
     ) -> bytes:
