@@ -1,5 +1,5 @@
 import sys as _sys
-from typing import Union
+import typing as _typing
 
 from cryptography.hazmat.backends import default_backend as _default_backend
 from cryptography.hazmat.primitives.ciphers import Cipher as _Cipher
@@ -18,6 +18,7 @@ __all__ = [
 
 def xor(data: bytes, key: bytes) -> bytes:
     r"""Apply "exlusive or" to two bytes instances.
+
     Many thanks:
     https://stackoverflow.com/a/29409299
 
@@ -42,6 +43,7 @@ def xor(data: bytes, key: bytes) -> bytes:
 
 def odd_parity(v: int) -> int:
     r"""Check integer parity.
+
     Many thanks: in_parallel
     http://p-nand-q.com/python/_algorithms/math/bit-parity.html
 
@@ -63,8 +65,8 @@ def odd_parity(v: int) -> int:
     return (0x6996 >> v) & 1
 
 
-def adjust_key_parity(key: Union[bytes, bytearray]) -> bytes:
-    r"""Adjust DES key parity key
+def adjust_key_parity(key: _typing.Union[bytes, bytearray]) -> bytes:
+    r"""Adjust DES key parity key.
 
     Parameters
     ----------
